#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""Text segmentation module

The textseg module provides functions fill() and wrap(), Unicode-aware
alternatives for those of textwrap standard module.  It also provides
classes LineBreak and GCStr, lower level interfaces for text segmentation.
"""

# Copyright (C) 2012 Hatuka*nezumi - IKEDA Soji.

__all__ = ['Consts', 'GCStr', 'LineBreak', 'fill', 'fold', 'wrap']

import re
import _textseg
from textseg.Consts import lbcBK, lbcCR, lbcLF, lbcNL, lbcSP, eawN

try:
    unicode, unichr
except NameError:
    unicode = str
    unichr = chr

###
### Function wrap()
###

def wrap(text,
         width = 70,
         initial_indent = "",
         subsequent_indent = "",
         expand_tabs = True,
         replace_whitespace = True,
         fix_sentence_endings = False, # currently no effects
         break_long_words = True,
         **kwds):
    '''\
wrap(text[, options...]) -> [unicode]

Wrap paragraphs of a text then return a list of wrapped lines.

Reformat each paragraph in ``text`` so that it fits in lines of no more than 
``width`` columns if possible, and return a list of wrapped lines.  By 
default, tabs in ``text`` are expanded and all other whitespace characters 
(including newline) are converted to space.'''

    def format(self, action, s):
        if action.startswith('eo'):
            return self.newline
        if action in ('sot', 'sop'):
            return s * 0 + initial_indent + s
        if action == 'sol':
            return s * 0 + subsequent_indent + s
        if action == '':
            if s == initial_indent or s == subsequent_indent:
                return ''
        return None

    if expand_tabs:
        text = GCStr(text).expandtabs()
    if replace_whitespace:
        table = {}
        for c in unicode('\t\n\x0b\x0c\r '):
            table[c] = unicode(' ')
        text = text.translate(table)

    for k, v in list({ 'charmax': 0,
                       'format': format,
                       'newline': '',
                       'urgent': (break_long_words and "FORCE" or None),
                       'width': width,
                     }.items()):
        kwds.setdefault(k, v)
    lb = LineBreak(**kwds)
    return [unicode(s) for s in lb.wrap(text)]

###
### Function fill()
###

def fill(text, **kwds):
    '''\
fill(text[, options...]) -> unicode

Reformat the single paragraph in ``text`` to fit in lines of no more than 
``width`` columns, and return a new string containing the entire wrapped 
paragraph.  Optional named arguments will be passed to wrap() function.'''

    return unicode("\n").join(wrap(text, **kwds))

###
### Function fold()
###

def _fold_FIXED(self, action, s):
    s = GCStr(s)
    if action in ('sot', 'sop'):
        self['_'] = {}
        self['_']['width'] = self.width
        if s.startswith('>'):
            self.width = 0
    elif action == '':
        self['_']['line'] = s
    elif action == 'eol':
        return self.newline
    elif action.startswith('eo'):
        if len(self['_']['line']) and self.width:
            s = self.newline + self.newline
        else:
            s = self.newline
        self.width = self['_']['width']
        del self['_']
        return s
    return None

_prefixRe = re.compile(r'^>+')
def _fold_FLOWED(self, action, s):
    s = GCStr(s)
    if action == 'sol':
        if len(self['_']['prefix']):
            return self['_']['prefix'] + ' ' + s
        elif s.startswith(' ') or s.startswith('From ') or s.startswith('>'):
            return ' ' + s
    elif action.startswith('so'):
        self['_'] = {}
        m = _prefixRe.match(unicode(s))
        if m:
            self['_']['prefix'] = m.group()
        else:
            self['_']['prefix'] = ''
            if s.startswith(' ') or s.startswith('From '):
                return ' ' + s
    elif action == '':
        self['_']['line'] = s
    elif action == 'eol':
        if len(s):
            s = ' '
        return s + ' ' + self.newline
    elif action.startswith('eo'):
        if len(self['_']['line']) and not len(self['_']['prefix']):
            s = ' ' + self.newline + self.newline
        else:
            s = self.newline
        del self['_']
        return s
    return None

def _fold_PLAIN(self, action, s):
    if action.startswith('eo'):
        return self.newline
    return None

_fold_funcs = {'flowed': _fold_FLOWED,
               'fixed': _fold_FIXED,
               'plain': _fold_PLAIN,
               }

_specialBreakRe = re.compile('([' + unichr(0xB) + unichr(0xC) +
                             unichr(0x85) + unichr(0x2028) + unichr(0x2029) +
                             '])', re.S)

def fold(string, method = 'plain', tabsize = 8, **kwds):
    """\
Folds lines of string *string* and returns it.

Surplus SPACEs and horizontal tabs at end of line are removed,
newline sequences are replaced by that specified by optional newline
argument and newline is appended at end of text if it does not exist.
Horizontal tabs are treated as tab stops according to tabsize argument.

Following options may be specified for ``method`` argument.

``"fixed"``
    Lines preceded by ">" won't be folded.
    Paragraphs are separated by empty line.
``"flowed"``
    "Format=Flowed; DelSp=Yes" formatting defined by RFC 3676.
``"plain"``
    Default method.  All lines are folded.
"""
    def sizing(self, cols, pre, spc, s):
        spcstr = spc + s
        i = 0
        for c in spcstr:
            if c.lbc != lbcSP:
                cols += spcstr[i:].cols
                break
            if c == "\t":
                if 0 < tabsize:
                    cols += tabsize - (cols % tabsize)
            else:
                cols += c.cols
            i = i + 1
        return cols

    if string is None or not len(string):
        return ''
    if not isinstance(string, unicode):
        string = unicode(string)

    kwds.update({'format': _fold_funcs.get(method.lower(), _fold_PLAIN),
                 'sizing': sizing,
                 })
    lb = LineBreak(**kwds)
    lbc = lb.lbc
    lbc[ord("\t")] = lbcSP
    lb.lbc = lbc

    result = ''
    for s in _specialBreakRe.split(string):
        if not len(s):
            continue
        elif _specialBreakRe.match(s):
            result += s
        else:
            result += ''.join([unicode(l) for l in lb.wrap(s)])
    return result

###
### Class for Line breaking
###

class LineBreak(_textseg.LineBreak):
    '''\
LineBreak class performs Line Breaking Algorithm described in Unicode 
Standards Annex #14 [UAX14]_. East_Asian_Width informative properties 
defined by Annex #11 [UAX11]_ will be concerned to determine breaking 
positions.
'''

    MANDATORY = 4
    DIRECT = 3
    INDIRECT = 2
    PROHIBITED = 1

    DEFAULTS = {
        'width': 70,
        'minwidth': 0,
        'charmax': 998,
        'eastasian_context': False,
        'eaw': None,
        'format': 'SIMPLE',
        'hangul_as_al': False,
        'lbc': None,
        'legacy_cm': True,
        'newline': "\n",
        'prep': None,
        'sizing': "UAX11",
        'urgent': None,
        'virama_as_joiner': True,
    }

    def __init__(self, **kwds):
        '''\
LineBreak([options...]) -> LineBreak

Create new LineBreak object.  Optional named arguments may specify 
initial property values.  See documentations of each properties.'''
        for k, v in list(self.DEFAULTS.items()):
            kwds.setdefault(k, v)
        _textseg.LineBreak.__init__(self, **kwds)

###
### Class for Grapheme cluster string
###

class GCStr(_textseg.GCStr):
    '''\
GCStr class treats Unicode string as a sequence of 
*extended grapheme clusters* defined by Unicode Standard Annex #29 
[UAX29]_.

**Grapheme cluster** is a sequence of Unicode character(s) that consists 
of one **grapheme base** and optional **grapheme extender** and/or 
**"prepend" character**.  It is close in that people consider as 
*character*.'''

    PROHIBIT_BEFORE = 1
    ALLOW_BEFORE = 2

    def __new__(cls, string, lb = None):
        '''\
GCStr(string[, lb]) -> GCStr

Create new grapheme cluster string (GCStr object) from Unicode string
``string``.

Optional LineBreak object ``lb`` controls breaking features.  Following
properties of LineBreak object affect new GCStr object.

    - eastasian_context
    - eaw
    - lbc
    - legacy_cm
    - virama_as_joiner
'''

        if lb is None:
            lb = LineBreak()
        return _textseg.GCStr.__new__(cls, string, lb=lb)

    '''
    def center(self, width, fillchar=' '):
        """\
S.center(width[, fillchar]) -> GCStr

Return S centered in a string of width columns. Padding is
done using the specified fill character (default is a space)"""

        fillchar = self * 0 + fillchar
        if width < self.cols + fillchar.cols:
            return self

        marg = (width - self.cols) // fillchar.cols
        right = marg // 2 + (marg & (width // fillchar.cols) & 1)
        return fillchar * (marg - right) + self + fillchar * right
    '''

    def endswith(self, suffix, start = 0, end = None):
        '''\
S.endswith(suffix[, start[, end]]) -> bool

Return True if S ends with the specified suffix, False otherwise.
With optional start, test S beginning at that position.
With optional end, stop comparing S at that position.
suffix can also be a tuple of strings to try.'''
        if isinstance(suffix, tuple):
            pass
        elif not isinstance(suffix, unicode):
            prefix = unicode(suffix)

        if end is None:
            return unicode(self[start:]).endswith(suffix)
        else:
            return unicode(self[start:end]).endswith(suffix)

    def expandtabs(self, tabsize=8):
        '''\
S.expandtabs([tabsize]) -> GCStr

Return a copy of S where all tab characters are expanded using spaces.
If *tabsize* is not given, a tab size of 8 columns is assumed.'''

        ret = self * 0
        j = 0
        for c in self:
            if c.lbc in (lbcBK, lbcCR, lbcLF, lbcNL):
                ret += c
                j = 0
            elif c == unicode('\t'):
                if 0 < tabsize:
                    incr = tabsize - (j % tabsize)
                    ret += unicode(' ') * incr
                    j += incr
            else:
                ret += c
                j += c.cols
        return ret

    def ljust(self, width, fillchar=' '):
        """\
S.ljust(width[, fillchar]) -> GCStr

Return S left-justified in a grapheme cluster string of width columns.
Padding is done using the specified fill character (default is a space)."""

        fillchar = self * 0 + fillchar
        if width < self.cols + fillchar.cols:
            return self
        return self + fillchar * ((width - self.cols) // fillchar.cols)

    def rjust(self, width, fillchar=' '):
        """\
S.rjust(width[, fillchar]) -> GCStr

Return S right-justified in a string of width columns. Padding is\n\
done using the specified fill character (default is a space)."""

        fillchar = self * 0 + fillchar
        if width < self.cols + fillchar.cols:
            return self
        return fillchar * ((width - self.cols) // fillchar.cols) + self

    def startswith(self, prefix, start = 0, end = None):
        '''\
S.startswith(prefix[, start[, end]]) -> bool

Return True if S starts with the specified prefix, False otherwise.
With optional start, test S beginning at that position.
With optional end, stop comparing S at that position.
prefix can also be a tuple of strings to try.'''
        if isinstance(prefix, tuple):
            pass
        elif not isinstance(prefix, unicode):
            prefix = unicode(prefix)

        if end is None:
            return unicode(self[start:]).startswith(prefix)
        else:
            return unicode(self[start:end]).startswith(prefix)

    def translate(self, table):
        '''\
S.translate(table) -> GCStr

Return a copy of the GCStr object S, where all characters have been mapped
through the given translation table, which must be a mapping of
Unicode ordinals to Unicode ordinals, strings, or None.
Unmapped characters are left untouched. Characters mapped to None
are deleted.'''
        return self * 0 + unicode(self).translate(table)




#! /usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import unittest
from textseg import LineBreak, fill, fold
from textseg.Consts import eawZ, eawN, lbcID, sea_support

try:
    unicode, unichr
except NameError:
    unicode = str
    unichr = chr

def unistr(list):
    return ''.join([unichr(c) for c in list])

LineBreak.DEFAULTS = {
    'charmax': 998,
    'eastasian_context': False,
    'eaw': { unistr([0x302E, 0x302F]): eawZ },
    'format': 'SIMPLE',
    'hangul_as_al': False,
    'lbc': None,
    'legacy_cm': True,
    'minwidth': 0,
    'newline': "\n",
    'prep': None,
    'sizing': "UAX11",
    'urgent': None,
    'virama_as_joiner': True,
    'width': 76,
}

SMALL_KANA = unistr([0x3041, 0x3043, 0x3045, 0x3047, 0x3049, 0x3063, 0x3083, 0x3085, 0x3087, 0x308E, 0x3095, 0x3096, 0x30A1, 0x30A3, 0x30A5, 0x30A7, 0x30A9, 0x30C3, 0x30E3, 0x30E5, 0x30E7, 0x30EE, 0x30F5, 0x30F6])

AMBIG_LATIN = unistr([0x00C6, 0x00D0, 0x00D8, 0x00DE, 0x00DF, 0x00E0, 0x00E1, 0x00E6, 0x00E8, 0x00E9, 0x00EA, 0x00EC, 0x00ED, 0x00F0, 0x00F2, 0x00F3, 0x00F8, 0x00F9, 0x00FA, 0x00FC, 0x00FE, 0x0101, 0x0111, 0x0113, 0x011B, 0x0126, 0x0127, 0x012B, 0x0131, 0x0132, 0x0133, 0x0138, 0x013F, 0x0140, 0x0141, 0x0142, 0x0144, 0x0148, 0x0149, 0x014A, 0x014B, 0x014D, 0x0152, 0x0153, 0x0166, 0x0167, 0x016B, 0x01CE, 0x01D0, 0x01D2, 0x01D4, 0x01D6, 0x01D8, 0x01DA, 0x01DC, 0x0251, 0x0261])

class LineBreakTest(unittest.TestCase):

    def readText(self, fn):
        fp = open(os.path.join('test-data', fn), 'rb')
        ret = unicode(fp.read(), 'utf-8')
        fp.close()
        return ret

    def doTest(self, pairs, **kwds):
        lb = LineBreak(**kwds)
        for infn, outfn in pairs:
            instring = self.readText(infn + '.in')
            b = lb.wrap(instring)
            broken = ''.join([unicode(x) for x in b])
            outstring = self.readText(outfn + '.out')
            self.assertEqual(broken, outstring)

    def test_00LineBreakTest(self):
        commentRe = re.compile(r'\s*#\s*')
        opRe = re.compile(r'\s*(?:' + unichr(0xF7) + '|' + unichr(0xD7) + r')\s*')

        try:
            fp = open(os.path.join('test-data', 'LineBreakTest.txt'), 'rb')
        except IOError:
            return

        eaw = {}
        for c in range(1, 0xFFFD):
            eaw[unichr(c)] = eawN
        lb = LineBreak(break_indent = False,
             width = 1,
             eaw = eaw,
             format = None,
             legacy_cm = False)

        print('')
        print(fp.readline().strip())
        print(fp.readline().strip())

        errs = 0
        tests = 0
        for l in fp.readlines():
            l = unicode(l, 'utf-8').rstrip()
            a = commentRe.split(l, 1)
            if len(a) > 1:
                desc = a[1]
            else:
                desc = ''
            l = a[0].strip()
            if not len(l):
                continue

            if l.startswith(unichr(0xD7)):      # ÷
                l = l[1:].lstrip()
            if l.endswith(unichr(0xF7)):        # ×
                l = l[:-1].rstrip()

            s = ''.join([unichr(int(c, 16))
                         for c in opRe.split(l)
                         if len(c) > 0])
            b = unistr([0x20, 0xF7, 0x20]).join([unistr([0x20, 0xD7, 0x20]).join(['%04X' % ord(c) for c in unicode(x)])
                                                for x in lb.wrap(s)])
            try:
                self.assertEqual(b, l)
            except AssertionError:
                import sys
                import traceback
                print('Failed: %s' % desc)
                traceback.print_exc(0)
                errs = errs + 1
            tests = tests + 1
        fp.close()

        if errs > 0:
            raise AssertionError("%d of %d subtests are failed." % (errs, tests))

    def test_01break(self):
        langs = ['ar', 'el', 'fr', 'he', 'ja', 'ja-a', 'ko', 'ru',
                 'vi', 'vi-decomp', 'zh']
        self.doTest([(x, x) for x in langs])

    def test_02hangul(self):
        self.doTest([('ko', 'ko.al')], hangul_as_al=True);
        self.doTest([('amitagyong', 'amitagyong')]);

    def test_03ns(self):
        self.doTest([('ja-k', 'ja-k')], width=72)
        self.doTest([('ja-k', 'ja-k.ns')],
                    lbc={ SMALL_KANA: lbcID }, width=72)

    def test_04fold(self):
        for lang in ('ja', 'fr', 'quotes'):
            instring = self.readText(lang + '.in')
            folded = {}
            for meth in ['plain', 'fixed', 'flowed']:
                folded[meth] = fold(instring, meth)
                outstring = self.readText(lang + '.' + meth + '.out')
                self.assertEqual(folded[meth], outstring)
            '''
            for meth in ['fixed', 'flowed']:
                outstring = unfold(folded[meth], meth)
                instring = self.readText(lang + '.' + 'norm.in')
                self.assertEqual(outstring, instring)
            '''

    def test_05urgent(self):
        self.doTest([('ecclesiazusae', 'ecclesiazusae')]);
        self.doTest([('ecclesiazusae', 'ecclesiazusae.ColumnsMax')],
                    urgent='FORCE')
        self.doTest([('ecclesiazusae', 'ecclesiazusae.CharactersMax')],
                    charmax=79)
        self.doTest([('ecclesiazusae', 'ecclesiazusae.ColumnsMin')],
                    minwidth=7, width=66, urgent='FORCE')
        try:
            self.doTest([('ecclesiazusae', 'ecclesiazusae')], urgent='RAISE')
        except:
            self.assertEqual(True, True)
        else:
            self.assertEqual(True, False)

    def test_06context(self):
        self.doTest([('fr', 'fr.ea')], eastasian_context=True)
        self.doTest([('fr', 'fr')], eastasian_context=True,
                    eaw={ AMBIG_LATIN: eawN })

    def test_07sea(self):
        if sea_support is not None:
            print(('\n# SA word segmentation supported: %s' % sea_support))
        else:
            print('\n# SA word segmentation not supported.')
            return
        self.doTest([('th', 'th')], complex_breaking=True)

    def test_09uri(self):
        self.doTest([('uri', 'uri.break')], width=1, prep=['BREAKURI'])
        self.doTest([('uri', 'uri.nonbreak')], width=1, prep=['NONBREAKURI'])

    def test_11format(self):
        def format(self, state, s):
            if state.startswith('so'):
                return s * 0 + '    ' + state + '>' + s
            if state.startswith('eo'):
                return '<' + state + "\n"
            return None

        self.doTest([(x, x + '.format') for x in ['fr', 'ja']],
                    format = format)
        self.doTest([(x, x + '.newline') for x in ['fr', 'ko']],
                    format = 'NEWLINE')
        self.doTest([(x, x + '.newline') for x in ['fr', 'ko']],
                    format = 'TRIM')

    def test_12fill(self):
        for lang in ['ja', 'fr']:
            instring = self.readText(lang + '.in')
            folded = fill(instring, width = 76, initial_indent = ' ' * 8,
                          subsequent_indent = ' ' * 4) + "\n"
            outstring = self.readText(lang + '.wrap.out').expandtabs()
            self.assertEqual(folded, outstring)

    '''
    def test_13flowedsp(self):
        for lang in ['flowedsp']:
            instring = self.readText(lang + '.in')
            unfolded = unfold(instring, 'flowedsp');
            outstring = self.readText(lang + '.out')
            self.assertEqual(unfolded, outstring)
    '''

    def test_14sea_al(self):
        self.doTest([('th', 'th.al')], complex_breaking=False)

    def doTestArray(self, pairs, **kwds):
        for infn, outfn in pairs:
            instring = self.readText(infn + '.in')
            lb = LineBreak(**kwds)
            broken = [unicode(x) for x in lb.wrap(instring)]
            fp = open(os.path.join('test-data', outfn + '.out'), 'rb')
            outstrings = [unicode(x, 'utf-8') for x in fp.readlines()]
            fp.close()
            self.assertEqual(broken, outstrings);

    def test_15array(self):
        # break
        self.doTestArray([(x, x) for x in ['ar', 'el', 'fr', 'ja', 'ja-a',
                                           'ko', 'ru', 'zh']])
        # urgent
        self.doTestArray([('ecclesiazusae', 'ecclesiazusae')])
        self.doTestArray([('ecclesiazusae', 'ecclesiazusae.ColumnsMax')],
                         urgent='FORCE')
        self.doTestArray([('ecclesiazusae', 'ecclesiazusae.CharactersMax')],
                         charmax=79)
        self.doTestArray([('ecclesiazusae', 'ecclesiazusae.ColumnsMin')],
                         minwidth=7, width=66, urgent='FORCE')
        try:
            self.doTestArray([('ecclesiazusae', 'ecclesiazusae')],
                             urgent='RAISE')
        except:
            # unittest module of Python 2.3.x does not have assertTrue()
            self.assertEqual(True, True)
        else:
            self.assertEqual(True, False)

        # format
        def format(self, stat, s):
            if stat.startswith('so'):
                return s * 0 + '    ' + stat + '>' + s
            if stat.startswith('eo'):
                return '<' + stat + "\n"
            return None

        self.doTestArray([(x, x + '.format') for x in ['fr', 'ja']],
                         format = format)
        self.doTestArray([(x, x + '.newline') for x in ['fr', 'ko']],
                         format = "NEWLINE")
        self.doTestArray([(x, x + '.newline') for x in ['fr', 'ko']],
                         format = "TRIM")

    def test_16regex(self):
        # Regex matching most of URL-like strings.
        urire = re.compile(r'''(?iux)\b(?:url:)?
        (?:[a-z][-0-9a-z+.]+://|news:|mailto:)
        [\x21-\x7E]+''')
        punctre = re.compile(r'\A[\".:;,>]+\Z', re.U)

        # Breaking URIs according to some CMoS rules.
        def breakURI(self, s):
            # 17.11 1.1: [/] ÷ [^/]
            # 17.11 2:   [-] ×
            # 6.17 2:   [.] ×
            # 17.11 1.2: ÷ [-~.,_?#%]
            # 17.11 1.3: ÷ [=&]
            # 17.11 1.3: [=&] ÷
            # Default:  ALL × ALL
            r = ''
            ret = []
            b = ''
            for c in s:
                if b == '':
                    r = c
                elif r.lower().endswith('url:'):
                    ret += [r]
                    r = c
                elif b in '/' and not c in '/' or \
                    not b in '-.' and c in '-~.,_?\#%=&' or \
                    b in '=&' or c in '=&':
                     if r != '':
                         ret += [r]
                     r = c
                else:
                    r += c
                b = c
            if r != '':
                ret += [r]

            # Won't break punctuations at end of matches.
            while 2 <= len(ret) and re.search(punctre, ret[-1]):
                c = ret[-1]
                ret = ret[0:-1]
                ret[-1] += c;
            return ret

        def nonBreak(self, s):
            return s

        # (REGEX, FUNC) pair
        self.doTest([('uri', 'uri.break')], width=1,
                    prep=[(urire, breakURI)])
        self.doTest([('uri', 'uri.nonbreak')], width=1,
                    prep=[(urire, nonBreak)])
        # [STRING, FUNC] pair
        self.doTest([('uri', 'uri.nonbreak')], width=1,
                     prep=[(urire.pattern, nonBreak)])
        # multiple patterns
        self.doTest([('uri', 'uri.break')], width=1,
                    prep=[(urire, breakURI),
                          (r'ftp://[\x21-\x7e]+', nonBreak)])
        self.doTest([('uri', 'uri.break.http')], width=1,
                    prep=[(r'ftp://[\x21-\x7e]+', nonBreak),
                          (urire, breakURI)])
        self.doTest([('uri', 'uri.nonbreak')], width=1,
                    prep=[(r'ftp://[\x21-\x7e]+', nonBreak),
                          (r'http://[\x21-\x7e]+', nonBreak),
                          (urire, breakURI)])


def suite():
    return unittest.makeSuite(LineBreakTest)

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())




#!/usr/bin/env python
# -*- coding: utf-8 -*-

def suite():
    #from textseg import *
    import unittest
    
    def my_import(name):
        # See http://docs.python.org/lib/built-in-funcs.html#l2h-6
        components = name.split('.')
        try:
            # python setup.py test
            mod = __import__(name)
            for comp in components[1:]:
                mod = getattr(mod, comp)
        except ImportError:
            # python tests/alltests.py
            mod = __import__(components[1])
        return mod

    modules_to_test = [
        'tests.LineBreakTest',
        'tests.GCStrTest',
    ]
    alltests = unittest.TestSuite()
    for module in map(my_import, modules_to_test):
        alltests.addTest(module.suite())
    return alltests


def dump_garbage():
    import gc
    print('\nGarbage:')
    gc.collect()
    if len(gc.garbage):
    
        print('\nLeaked objects:')
        for x in gc.garbage:
            s = str(x)
            if len(s) > 77: s = s[:73]+'...'
            print((type(x), '\n  ', s))
    
        print(('There were %d leaks.' % len(gc.garbage)))
    else:
        print('Python garbage collector did not detect any leaks.')
        print('However, it is still possible there are leaks in the C code.')


def runall(report_leaks=0):
    report_leaks = report_leaks
    
    if report_leaks:
        import gc
        gc.enable()
        gc.set_debug(gc.DEBUG_LEAK & ~gc.DEBUG_SAVEALL)
    
    import unittest
    
    try:
        unittest.TextTestRunner(verbosity=2).run(suite())
    finally:
        pass

    if report_leaks:
        dump_garbage()
    

if __name__ == '__main__':
    runall(1)




#! /usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import unittest
from textseg import GCStr, LineBreak
from textseg.Consts import lbcAL, lbcGL

try:
    unicode, unichr
except NameError:
    unicode = str
    unichr = chr

def unistr(list):
    return ''.join([unichr(c) for c in list])

class GCStrTest(unittest.TestCase):

    def test_00GraphemeBreakTest(self):
        commentRe = re.compile(r'\s*#\s*')
        opRe = re.compile(r'\s*(?:' + unichr(0xF7) + '|' + unichr(0xD7) +
                          r')\s*')

        try:
            fp = open(os.path.join('test-data', 'GraphemeBreakTest.txt'), 'rb')
        except IOError:
            return

        print('')
        print(fp.readline().strip())
        print(fp.readline().strip())

        errs = 0
        tests = 0
        for l in fp.readlines():
            l = unicode(l, 'utf-8').rstrip()
            a = commentRe.split(l, 1)
            if len(a) > 1:
                desc = a[1]
            else:
                desc = ''
            l = a[0].strip()
            if not len(l):
                continue

            if l.startswith(unichr(0xF7)):      # ÷
                l = l[1:].lstrip()
            if l.endswith(unichr(0xF7)):        # ×
                l = l[:-1].rstrip()

            s = ''.join([unichr(int(c, 16))
                         for c in opRe.split(l)
                         if len(c) > 0])
            b = unistr([0x20, 0xF7, 0x20]).join([unistr([0x20, 0xD7, 0x20]).join(['%04X' % ord(c) for c in unicode(x)])
                                                 for x in GCStr(s)])
            try:
                self.assertEqual(b, l)
            except AssertionError:
                import sys
                import traceback
                print('Failed: %s' % desc)
                traceback.print_exc(0)
                errs = errs + 1
            tests = tests + 1
        fp.close()

        if errs > 0:
            raise AssertionError("%d of %d subtests are failed." % (errs, tests))

    def test_10gcstring01(self):
        s = unistr([0x300, 0, 0x0D, 0x41, 0x300, 0x301, 0x3042, 0xD, 0xA,
                    0xAC00, 0x11A8])
        r = unistr([0xAC00, 0x11A8, 0xD, 0xA, 0x3042, 0x41, 0x300, 0x301,
                    0xD, 0, 0x300])
        string = GCStr(s)
        self.assertEqual(len(string), 7)
        self.assertEqual(string.cols, 5)
        self.assertEqual(string.chars, 11)
        l = [unicode(c) for c in string]
        l.reverse()
        self.assertEqual(r, ''.join(l))

    def test_10gcstring02(self):
        string = GCStr(unistr([0x1112, 0x1161,
                               0x11AB, 0x1100, 0x1173, 0x11AF]))
        self.assertEqual(len(string), 2)
        self.assertEqual(string.cols, 4)
        self.assertEqual(string.chars, 6)
        self.assertEqual(string, string.__copy__())

    def test_10gcstring03(self):
        string = GCStr(unistr([0x1112, 0x1161,
                               0x11AB, 0x1100, 0x1173, 0x11AF]))
        s1 = unistr([0x1112, 0x1161])
        s2 = unistr([0x11AB, 0x1100, 0x1173, 0x11AF])
        g1 = GCStr(s1)
        g2 = GCStr(s2)
        self.assertEqual(g1 + g2, string)
        self.assertEqual(len(g1 + g2), 2)
        self.assertEqual((g1 + g2).cols, 4)
        self.assertEqual(string.chars, 6)
        self.assertEqual(g1 + s2, string)
        self.assertEqual(len(g1 + s2), 2)
        self.assertEqual((g1 + s2).cols, 4)
        self.assertEqual(string.chars, 6)
        self.assertEqual(s1 + g2, string)
        self.assertEqual(len(s1 + g2), 2)
        self.assertEqual((s1 + g2).cols, 4)
        self.assertEqual(string.chars, 6)
        s1 += g2
        self.assertEqual(s1, string)
        g1 += s2
        self.assertEqual(g1, string)

    def test_10gcstring04(self):
        string = GCStr(unistr([0x1112, 0x1161,
                               0x11AB, 0x1100, 0x1173, 0x11AF]))
        self.assertEqual(unicode(string[1:]),
                         unistr([0x1100, 0x1173, 0x11AF]))
        self.assertEqual(unicode(string[-1:]),
                         unistr([0x1100, 0x1173, 0x11AF]))
        self.assertEqual(string[0:-1],
                         unistr([0x1112, 0x1161, 0x11AB]))
        string[-1:] = "A"
        self.assertEqual(unicode(string),
                         unistr([0x1112, 0x1161, 0x11AB, 0x41]))
        string[2:] = "B"
        self.assertEqual(string,
                         unistr([0x1112, 0x1161, 0x11AB, 0x41, 0x42]))
        string[0:0] = "C"
        self.assertEqual(unicode(string),
                         unistr([0x43, 0x1112, 0x1161, 0x11AB, 0x41, 0x42]))

    def test_10gcstring05(self):
        s = [unistr([0x0300]), unistr([0x00]), unistr([0x0D]),
             unistr([0x41, 0x0300, 0x0301]), unistr([0x3042]),
             unistr([0x0D, 0x0A]), unistr([0xAC00, 0x11A8])]
        string = GCStr(''.join(s))
        self.assertEqual([unicode(c) for c in string], s)

    def test_17prop(self):
        lb = LineBreak(eastasian_context = True)

        for s in [unichr(0xA0)]:
            g = GCStr(s, lb)
            self.assertEqual(len(g), 1)
            self.assertEqual(g.lbc, lbcGL)

        g = GCStr(unistr([0xC2, 0xA0]), lb)
        self.assertEqual(len(g), 2)
        self.assertEqual(g.lbc, lbcAL)

        for s in [unichr(0xD7)]:
            g = GCStr(s, lb)
            self.assertEqual(len(g), 1)
            self.assertEqual(g.cols, 2)

        g = GCStr(unistr([0xC3, 0x97]), lb)
        self.assertEqual(len(g), 2)
        self.assertEqual(g.cols, 1)


def suite():
    return unittest.makeSuite(GCStrTest)

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())




# -*- python -*-
# -*- coding: utf-8 -*-
"""\
Constants for textseg package.

_`eawNa`, _`eawN`, _`eawA`, _`eawW`, _`eawH`, _`eawF`, _`eawZ`
    Index values to specify 6 East_Asian_Width properties
    defined by [UAX #11], and EA_Z to specify nonspacing.

_`lbcBK`, _`lbcCR`, _`lbcLF`, _`lbcNL`, _`lbcSP`, _`lbcOP`, _`lbcCL`, _`lbcCP`, _`lbcQU`, _`lbcGL`, _`lbcNS`, _`lbcEX`, _`lbcSY`, _`lbcIS`, _`lbcPR`, _`lbcPO`, _`lbcNU`, _`lbcAL`, _`lbcHL`, _`lbcID`, _`lbcIN`, _`lbcHY`, _`lbcBA`, _`lbcBB`, _`lbcB2`, _`lbcCB`, _`lbcZW`, _`lbcCM`, _`lbcWJ`, _`lbcH2`, _`lbcH3`, _`lbcJL`, _`lbcJV`, _`lbcJT`, _`lbcSG`, _`lbcAI`, _`lbcCJ`, _`lbcSA`, _`lbcXX`
    Index values to specify 39 line breaking properties (classes)
    defined by [UAX #14].

    .. note::
       Property value ``CP`` was introduced by Unicode 5.2.0.
       Property value ``HL`` and ``CJ`` may be introduced by Unicode 6.1.0.

_`sea_support`
    Flag to determin if word segmentation for South East Asian writing
    systems is enabled.  If this feature was enabled, a non-empty string
    is set.  Otherwise, ``None`` is set.

    .. note::
       Current release supports Thai script of modern Thai language only.

_`unicode_version`
    A string to specify version of Unicode standard this module refers.
"""

from _textseg_Consts import *




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 Hatuka*nezumi - IKEDA Soji
#
# This file is part of pytextseg.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import re
from glob import glob

try:
    from distutils2.core import setup, Extension
    from distutils2.command.build_clib import build_clib
except ImportError:
    try:
        from setuptools import setup, Extension
        from setuptools.command.build_clib import build_clib
    except ImportError:
        from distutils.core import setup, Extension
        from distutils.command.build_clib import build_clib

###############################################################################
# CONSTANTS
###############################################################################

SOMBOK_VERSION = '2.1.1'
UNICODE_VERSION = '6.1.0'
LIBTHAI_VERSION = '0.1.9'
PKG_CONFIG = os.environ.get('PKG_CONFIG', 'pkg-config')

###############################################################################
# UTILS
###############################################################################
try:
    from subprocess import getstatusoutput
except ImportError:
    from commands import getstatusoutput

def pkg_config(*packages, **kwds):
    flag_map = {'-I': 'include_dirs', '-L': 'library_dirs', '-l': 'libraries'}

    pkgs = ' '.join(["'%s'" % p for p in packages])
    stat, out = getstatusoutput("%s --libs --cflags %s" % (PKG_CONFIG, pkgs))
    if stat >> 8:
        raise RuntimeError(out)
    for token in out.split():
        kwds.setdefault(flag_map.get(token[:2]), []).append(token[2:])
    return kwds

###############################################################################
# CUSTOMIZE
###############################################################################
class my_build_clib(build_clib):
    def run(self):
        include_dir = os.path.join('sombok', 'include')

        h = open(os.path.join(include_dir, 'sombok.h.in'), 'rt').read()
        h = h.replace('#ifdef HAVE_CONFIG_H', '#if 1')
        try:
            pkg_config('libthai >= %s' % LIBTHAI_VERSION)
            print("# libthai support is enabled.")
            h = h.replace('"config.h"', '''\
<Python.h>
#    define USE_LIBTHAI "libthai-unknown"''')
        except RuntimeError:
            print("# libthai support is disabled.  You might want to install it. and to try again.")
            h = h.replace('"config.h"', '''\
<Python.h>
#    undef USE_LIBTHAI''')
        h = h.replace('@SOMBOK_UNICHAR_T@', 'Py_UCS4')
        h = h.replace('@PACKAGE_VERSION@', 'bundled')
        h = h.replace('@SOMBOK_UNICHAR_T_IS_WCHAR_T@', '')
        h = h.replace('@SOMBOK_UNICHAR_T_IS_UNSIGNED_INT@', '')
        h = h.replace('@SOMBOK_UNICHAR_T_IS_UNSIGNED_LONG@', '')
        sombok_h = open(os.path.join(include_dir, 'sombok.h'), 'wt')
        sombok_h.write(h)
        sombok_h.close()

        build_clib.run(self)

###############################################################################
# MAIN
###############################################################################
if __name__ == '__main__':

    try:
        ext_config = pkg_config('libthai >= %s' % LIBTHAI_VERSION,
                                include_dirs = ['.'])
    except RuntimeError:
        ext_config = {'include_dirs': ['.'], }
    libraries = []
    try:
        ext_config = pkg_config('sombok >= %s' % SOMBOK_VERSION, **ext_config)
    except RuntimeError:
        from distutils.sysconfig import get_python_inc

        sources = [s for s in glob(os.path.join('sombok', 'lib', '*.c'))
                     if not re.search(r'[0-9][.0-9]+\.c', s)]
        sources.append(os.path.join('sombok', 'lib', '%s.c' % UNICODE_VERSION))
        include_dir = os.path.join('sombok', 'include')

        libraries.append(('bundled_sombok',
                          {'sources': sources,
                           'include_dirs': [include_dir,
                                            get_python_inc(plat_specific=1)],
                          },
                         ))
        ext_config['include_dirs'].append(include_dir)

    setup(
        name="pytextseg",
        version='0.0.4',
        license="GNU General Public License (GPL)",
        description='Python module for text segmentation.',
        long_description=open('README', 'r').read(),
        author='Hatuka*nezumi - IKEDA Soji',
        author_email='hatuka@nezumi.nu',
        url="http://pypi.python.org/pypi/pytextseg/",
        download_url = '',
        platforms=["any"],
        classifiers=[
            "Development Status :: 3 - Alpha",
	    "Environment :: Other Environment",
	    "Intended Audience :: Developers",
	    "License :: OSI Approved :: GNU General Public License (GPL)",
	    "Operating System :: OS Independent",
	    "Programming Language :: Python",
	    "Programming Language :: Python :: 2",
	    "Programming Language :: Python :: 3",
	    "Programming Language :: C",
	    "Topic :: Software Development :: Internationalization",
	    "Topic :: Software Development :: Libraries :: Python Modules",
	    "Topic :: Text Processing :: General",
	    "Topic :: Text Processing :: Linguistic"
	],
        scripts=[],
	packages=['textseg'],
	ext_modules=[
	    Extension(
		'_textseg',
		['textseg.c'],
		**ext_config
	    ),
	    Extension(
		'_textseg_Consts',
		['textseg_Consts.c'],
		**ext_config
	    ),
	],
        libraries=libraries,
        cmdclass={'build_clib': my_build_clib, },
    )

